function [xTN,txtTN] = StataToMLPanelDataPs(Filename,colID,colDateX,colDateTxt,DateFormat, ...
                                          nCols,vvStr, ...
                                          StartRow,delim,vvKeep,nLines,NumFmt0);
%StataToMLPanelDataPs    Converts panel data set exported from Stata (text) to a TxN matlab matrix
%                        (for the numeric part) and a cell array (for the text part)
%
%
%
%
%  Usage:  [xTN,txtTN] = StataToMLPanelDataPs(Filename,colID,colDateX,colDateTxt,DateFormat, ...
%                                           nCols,vvStr[,...
%                                           StartRow[,delim[,vvKeep[,nLines[,NumFmt0]]]]]);
%
%
%  Input:          Filename       string, filename, eg 'StataToMLPanelData_Data.csv'
%                  colID          scalar, column number with ID variable, eg 1
%                  colDateX       scalar, column number (in x) of date variable,
%                                 use [] if date is among string variables
%                  colDateTxt     scalar, column number (in xtxt) of string date variable,
%                                 use [] if date is among numerical variables, eg. 1
%                  DateFormat     string, date format, eg 'mm/dd/yyyy'; irrelevant if dates are numerical
%                  nCols          scalar, number of columns in file, eg 4
%                  vvStr          vector, number of columns with string variables, eg [2,4]
%                  StartRow       (optional) scalar, which row in file to start reading in, [2]
%                  delim          (optional) string, delimiter, [',']
%                  vvKeep         (optional) vector of columns to keep, [[], meaning all columns]
%                  nLines         (optional) scalar, number of lines to read in one chunk, [1000];
%                  NumFmt0        (optional) string, number format, ['%n']
%
%  Output:         xTN            T x N x Kx matrix with numerical data
%                  txtTN          T x N x Ktxt cell array with string data
%
%
%
%  Calls on:  TextScan2Ps, FindMPs
%
%
%
%  Paul.Soderlind@unisg.ch   19 December 2011
%------------------------------------------------------------------------------

if nargin < 8;
  StartRow = 2;
end;
if nargin < 9;
  delim = ',';
end;
if nargin < 10;
  vvKeep = [];
end;
if nargin < 11;
  nLines = 1000;
end;
if nargin < 12;
  NumFmt0 = '%n';
end;

                                                        %read in file
[xM,txtM,x_headers,txt_headers] = TextScan2Ps(Filename,nCols,vvStr, ...
                                             StartRow,delim,vvKeep,nLines,NumFmt0);


if (isscalar(colDateTxt) && ~isnan(colDateTxt));          %dates, if date is in txtM
  dStr               = char(txtM(:,colDateTxt));
  txtM(:,colDateTxt) = [];
  dN_M               = datenum(dStr,DateFormat);
  clear dStr;
  ID_M        = xM(:,colID);                 %IDs of individuals
  xM(:,colID) = [];
elseif (isscalar(colDateX) && ~isnan(colDateX));   %dates, if date is in xM
  dN_M                   = xM(:,colDateX);
  ID_M                   = xM(:,colID);                
  xM(:,[colDateX,colID]) = [];
end;
dN = unique(dN_M);
T  = length(dN);


ID          = unique(ID_M);
N           = length(ID);

Kx          = size(xM,2);                  %number of variables in numeric matrix
Ktxt        = size(txtM,2);                %number of variables in text part


xTN   = NaN(T,N,Kx);                       %TxNxK matrix of individual data (t,i,k variables)
txtTN = cell(T,N,Ktxt);

for i = 1:length(ID);                      %loop over individuals
  vvi  = ID_M == ID(i);                    %logicals of obs for ID(i)
  vvti = FindMPs(dN_M(vvi),dN);            %indices in dN of those dates
  if Kx >= 1;                              %if there is numerical data
    xi            = xM(vvi,:);             %pick out data for ID(i)
    xTN(vvti,i,:) = xi;                    %pick obs for ID(i) in column i and correct rows
  end;
  if Ktxt >= 1;                            %if there is string data
    txti            = txtM(vvi,:);
    txtTN(vvti,i,:) = txti;
  end;
  if mod(i,1000) == 0;
    fprintf('%s','.');
  end;
  clear xi txti vvi vvti;
end;
%------------------------------------------------------------------------------
