function [xM,txtM,x_headers,txt_headers] = TextScan2Ps(Filename,nCols,vvStr, ...
                                             StartRow,delim,vvKeep,nLines,NumFmt0);
%  TextScan2Ps    Read csv/txt file with some numerical columns and some string columns. Uses (...,'CollectOutput',true)
%
%
%
%
%  Usage:    [x,txt,x_headers,txt_headers] = TextScan2Ps(Filename,nCols,vvStr, ...
%                                              StartRow,delim,vvKeep,nLines,NumFmt0);
%
%  Input:    Filename       string, file name
%            nCols          scalar, number of columns in data set
%            vvStr          vector, indices of columns with string data
%            StartRow       (optional) scalar, index of row number where the main data starts (2 if one line of headers) [2]
%            delim          (optional) string, delimiter, eg. ',' for csv files, '\t' for tab separated files, else [] (for
%                           the default white space delimiter, ie. spaces and tabs). If a nondefault delimiter is specified,
%                           repeated delimiter characters are treated as separate delimiters, [[]]
%            vvKeep         (optional) vector, indices of columns to keep and export [1:nCols]. Notice that
%                           xM contains the columns in the same order as in the file: [2,1] and [1,2] give
%                           the same output. (Effectively, sort(vvKeep) is used.)
%            nLines         (optional) scalar, number of lines to read in one chunk [1000]
%            NumFmt0        (optional), string, eg. '%f32' (single prec float) or '%n' ['%n', double precision float]
%
%  Output:   xM             Txn1 matrix with numerical data
%            txtM           Txn2 cell array with string data
%            x_headers      (StartRow-1)xn1 cell array with headers for numerical columns
%            txt_headers    (StartRow-1)xn2 cell array with headers for string columns
%
%
%
%
%
%  Paul.Soderlind@unisg.ch   Sep 2006, Jan 2013
%------------------------------------------------------------------------------

if (nargin < 4) || isempty(StartRow);
  StartRow = 2;
end;
if (nargin < 5);
  delim = [];
end;
if (nargin < 6) || isempty(vvKeep);
  vvKeep = 1:nCols;
end;
if (nargin < 7) || isempty(nLines);
  nLines = 1000;
end;
if (nargin < 8) || isempty(NumFmt0);
  NumFmt0 = '%n';
end;


vvNum = setdiff(1:nCols,vvStr);        %indices of the columns with numerical data


fmtNum  = '';                             %format for numerical part of file
fmtNumH = '';
for i = 1:nCols;
  if any(i==vvNum) && any(i==vvKeep);
    fmtNum  = [fmtNum  ' ' NumFmt0];          %%n
    fmtNumH = [fmtNumH ' %s'];
  elseif any(i==vvNum);
    fmtNum  = [fmtNum  ' %*n'];
    fmtNumH = [fmtNumH ' %*s'];
  else;
    fmtNum  = [fmtNum  ' %*s'];
    fmtNumH = [fmtNumH ' %*s'];
  end;
end;

fmtStr  = '';                             %format for string part of file
fmtStrH = '';
for i = 1:nCols;
  if any(i==vvStr) && any(i==vvKeep);
    fmtStr  = [fmtStr  ' %s'];
    fmtStrH = [fmtStrH ' %s'];
  elseif any(i==vvStr);
    fmtStr  = [fmtStr  ' %*s'];
    fmtStrH = [fmtStrH ' %*s'];
  else;
    fmtStr  = [fmtStr  ' %*n'];
    fmtStrH = [fmtStrH ' %*s'];
  end;
end;

fil1 = fopen(Filename);                %open file


if ~isempty(intersect(vvKeep,vvStr));      %only if we want to keep string columns
  nStr = length(intersect(vvKeep,vvStr));  %number of string columns to be exported

  txt = 'a';
  txtM = cell(1,nStr);
  if isempty(delim);
    txt_headers = textscan(fil1,fmtStrH,StartRow-1,'CollectOutput',true);
  else;
    txt_headers = textscan(fil1,fmtStrH,StartRow-1,'delimiter',delim,'CollectOutput',true);
  end;
  txt_headers = txt_headers{:};
  while ~isempty(txt);                 %read nLines, until no more data
    if isempty(delim);
      x2 = textscan(fil1,fmtStr,nLines);
    else;
      x2 = textscan(fil1,fmtStr,nLines,'delimiter',delim,'CollectOutput',true);
    end;
    if isempty(x2);
      x2 = cell(1,1);                          %fix for Octave, Jan 2013
    end;
    txt  = x2{:};
    if ~isempty(txt);
      txtM = [txtM;txt];
    end;
  end;
  frewind(fil1);                               %rewind to beginning of file
  clear('x2');
  txtM = txtM(2:end,:);                        %get rid of empty first row

else;

  txt_headers = [];
  txtM        = [];

end;


if ~isempty(intersect(vvKeep,vvNum));  %only if we want to keep numerical columns

  x2{1} = 1;
  xM = [];
  if isempty(delim);
    x_headers = textscan(fil1,fmtNumH,StartRow-1,'CollectOutput',true);
  else;
    x_headers = textscan(fil1,fmtNumH,StartRow-1,'delimiter',delim,'CollectOutput',true);
  end;
  x_headers = x_headers{:};
  while ~isempty(x2{:});                 %read nLines, until no more data
    if isempty(delim);
      x2 = textscan(fil1,fmtNum,nLines,'CollectOutput',true);
    else;
      x2 = textscan(fil1,fmtNum,nLines,'delimiter',delim,'CollectOutput',true);
    end;
    if isempty(x2);                            %fix for Octave, Jan 2013
      x2 = cell(1,1);
    end;
    x  = cell2mat(x2);
    xM = [xM;x];
  end;
  clear('x2');

else;

  x_headers = [];
  xM        = [];

end;

fclose(fil1);                  %close file
%------------------------------------------------------------------------------

